<?php

namespace Drupal\opensearch_views;

use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Manages field information from OpenSearch indices.
 */
class FieldManager {

  /**
   * The OpenSearch client service.
   *
   * @var \Drupal\opensearch_views\OpenSearchClient
   */
  protected $client;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * A cache of discovered fields, keyed by index name.
   *
   * @var array
   */
  protected $fields = [];

  /**
   * Constructor.
   *
   * @param \Drupal\opensearch_views\OpenSearchClient $client
   *   The OpenSearch client service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(OpenSearchClient $client, LoggerChannelFactoryInterface $logger_factory = NULL) {
    $this->client = $client;
    $this->logger = $logger_factory ? $logger_factory->get('opensearch_views') : NULL;
  }

  /**
   * Get all available fields for an index.
   *
   * @param string $index
   *   The index name.
   *
   * @return array
   *   An array of field information.
   */
  public function getFields($index) {
    if (empty($index)) {
      if ($this->logger) {
        $this->logger->warning('Attempted to get fields with empty index name');
      }
      return [];
    }

    if (!isset($this->fields[$index])) {
      $this->fields[$index] = $this->discoverFields($index);
    }
    
    return $this->fields[$index];
  }

  /**
   * Discover available fields in an index.
   *
   * @param string $index
   *   The index name.
   *
   * @return array
   *   An array of field information.
   */
  protected function discoverFields($index) {
    $fields = [];
    
    try {
      // Verify index exists first
      if (!$this->client->indexExists($index)) {
        if ($this->logger) {
          $this->logger->warning('Index @index does not exist', ['@index' => $index]);
        }
        return [];
      }
      
      // Get sample documents for field discovery
      $search_params = [
        'size' => 5, // Get multiple documents to improve field coverage
        'sort' => ['_doc'], // Sort by document order for efficiency
      ];
      
      $search_result = $this->client->search($index, $search_params);
      
      if (!empty($search_result['hits']['hits'])) {
        if ($this->logger) {
          $this->logger->info('Found @count sample documents for field discovery', 
            ['@count' => count($search_result['hits']['hits'])]);
        }
        
        // Process multiple documents to get more complete field coverage
        foreach ($search_result['hits']['hits'] as $hit) {
          if (!empty($hit['_source'])) {
            // Start extraction with an empty prefix
            $this->recursivelyExtractFields($hit['_source'], $fields);
          }
        }
      } 
      else {
        if ($this->logger) {
          $this->logger->warning('No documents found in index @index', ['@index' => $index]);
        }
      }
      
      // If we still have no fields, try to get mappings as a fallback
      if (empty($fields)) {
        $mappings = $this->client->getMappings($index);
        if (!empty($mappings[$index]['mappings']['properties'])) {
          if ($this->logger) {
            $this->logger->info('Found mappings for index @index', ['@index' => $index]);
          }
          $this->extractFieldsFromMappings($mappings[$index]['mappings']['properties'], $fields);
        }
      }
    }
    catch (\Exception $e) {
      if ($this->logger) {
        $this->logger->error('Error discovering fields for index @index: @message', [
          '@index' => $index,
          '@message' => $e->getMessage(),
        ]);
      }
      // In case of error, return an empty array.
      return [];
    }
    
    if ($this->logger) {
      $this->logger->info('Discovered @count fields for index @index', [
        '@count' => count($fields),
        '@index' => $index,
      ]);
    }
    
    // Add standard fields always
    $fields['_id'] = [
      'path' => '_id',
      'type' => 'string',
      'label' => 'Document ID (_id)',
    ];
    
    $fields['_source'] = [
      'path' => '_source',
      'type' => 'object',
      'label' => 'Full Document (_source)',
    ];
    
    return $fields;
  }

  /**
   * Recursively extract fields from a JSON document with special handling for complex structures.
   *
   * @param mixed $data
   *   The data to extract fields from.
   * @param array &$fields
   *   The array to populate with field information.
   * @param string $path
   *   The current path prefix.
   */

    protected function recursivelyExtractFields($data, array &$fields, $path = '') {
        // Base case: If data is not traversable, stop.
        if (!is_array($data) && !is_object($data)) {
            return;
        }

        // Convert objects to arrays for consistent key access.
        $data = (array) $data;

        foreach ($data as $key => $value) {
            // Skip keys starting with _ (often internal OpenSearch fields).
            if (strpos((string) $key, '_') === 0) {
                continue;
            }

            // Construct the path for the current key.
            $current_path = empty($path) ? (string) $key : $path . '.' . (string) $key;
            $valueType = $this->getValueType($value);

            // Add/overwrite the current field path to our list.
            $fields[$current_path] = [
                'path' => $current_path,
                'type' => $valueType,
                'label' => $current_path,
            ];

            // Decide whether to recurse further down.
            if ($valueType === 'object') {
                // Standard object/associative array recursion.
                $this->recursivelyExtractFields($value, $fields, $current_path);
            }
            elseif ($valueType === 'array') {
                // --- MODIFICATION START: Iterate first few array elements ---
                // Define how many array elements to sample (adjust if needed).
                $max_array_elements_to_sample = 3;
                $processed_elements = 0;

                // Loop through the elements of the sequential array.
                foreach ($value as $index => $element) {
                    // Stop after checking the first few elements.
                    if ($processed_elements >= $max_array_elements_to_sample) {
                        if ($this->logger) {
                           // Log that we stopped sampling array elements early
                           // $this->logger->debug('Stopped sampling elements for array path @path after @count elements.', ['@path' => $current_path, '@count' => $max_array_elements_to_sample]);
                        }
                        break;
                    }

                    // Ensure the index is numeric for sequential arrays (sanity check)
                    if (!is_numeric($index)) {
                        // This case shouldn't happen if getValueType correctly identified 'array'
                        continue;
                    }

                    // Construct path for the current element using dot notation.
                    $element_path = $current_path . '.' . $index;
                    $elementType = $this->getValueType($element);

                    // Add the path to the element itself.
                    $fields[$element_path] = [
                        'path' => $element_path,
                        'type' => $elementType,
                        'label' => $element_path,
                    ];

                    // Recurse *into* this element if it's complex.
                    if ($elementType === 'object' || $elementType === 'array') {
                        $this->recursivelyExtractFields($element, $fields, $element_path);
                    }

                    $processed_elements++;
                }
                // --- MODIFICATION END ---
            }
            // No recursion needed for scalar types.
        }
    }
    
  /**
   * Get the data type of a value.
   *
   * @param mixed $value
   *   The value to check.
   *
   * @return string
   *   The type as a string.
   */
  protected function getValueType($value) {
    if (is_null($value)) {
      return 'null';
    }
    
    if (is_bool($value)) {
      return 'boolean';
    }
    
    if (is_int($value)) {
      return 'integer';
    }
    
    if (is_float($value)) {
      return 'float';
    }
    
    if (is_string($value)) {
      return 'string';
    }
    
    if (is_array($value)) {
      return isset($value[0]) ? 'array' : 'object';
    }
    
    if (is_object($value)) {
      return 'object';
    }
    
    return 'unknown';
  }

  /**
   * Extract fields from index mappings.
   *
   * @param array $properties
   *   The properties from mappings.
   * @param array &$fields
   *   The fields array to populate.
   * @param string $prefix
   *   The field path prefix.
   */
  protected function extractFieldsFromMappings(array $properties, array &$fields, $prefix = '') {
    foreach ($properties as $name => $property) {
      $field_path = $prefix ? "$prefix.$name" : $name;
      
      if (!empty($property['properties'])) {
        // This is an object type.
        $fields[$field_path] = [
          'path' => $field_path,
          'type' => 'object',
          'label' => $field_path,
        ];
        
        // Recursively process the nested properties.
        $this->extractFieldsFromMappings($property['properties'], $fields, $field_path);
      }
      elseif (!empty($property['type']) && $property['type'] === 'nested') {
        // This is a nested object.
        $fields[$field_path] = [
          'path' => $field_path,
          'type' => 'nested',
          'label' => $field_path,
        ];
        
        if (!empty($property['properties'])) {
          $this->extractFieldsFromMappings($property['properties'], $fields, $field_path);
        }
      }
      else {
        // This is a field.
        $type = $property['type'] ?? 'string';
        
        $fields[$field_path] = [
          'path' => $field_path,
          'type' => $type,
          'label' => $field_path,
        ];
      }
    }
  }

  /**
   * Get field options for form select elements.
   *
   * @param string $index
   *   The index name.
   *
   * @return array
   *   An array of options for form select elements.
   */
  public function getFieldOptions($index) {
    $fields = $this->getFields($index);
    $options = [];
    
    foreach ($fields as $path => $field) {
      // Format the label to show the field type
      $type_label = !empty($field['type']) ? " (" . $field['type'] . ")" : '';
      $options[$path] = $path . $type_label;
    }
    
    // Sort options alphabetically for easier navigation
    ksort($options);
    
    return $options;
  }
}
