<?php

namespace Drupal\opensearch_views\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure OpenSearch Views settings.
 */
class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'opensearch_views_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['opensearch_views.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('opensearch_views.settings');

    $form['connection'] = [
      '#type' => 'details',
      '#title' => $this->t('OpenSearch Connection'),
      '#open' => TRUE,
    ];

    $form['connection']['host'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Host'),
      '#default_value' => $config->get('host') ?: 'localhost',
      '#required' => TRUE,
      '#description' => $this->t('The hostname or IP address of your OpenSearch server.'),
    ];

    $form['connection']['port'] = [
      '#type' => 'number',
      '#title' => $this->t('Port'),
      '#default_value' => $config->get('port') ?: 9200,
      '#required' => TRUE,
      '#min' => 1,
      '#max' => 65535,
      '#description' => $this->t('The port of your OpenSearch server.'),
    ];

    $form['connection']['username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Username'),
      '#description' => $this->t('Username for authentication (if required).'),
    ];

    $form['connection']['password'] = [
      '#type' => 'password',
      '#title' => $this->t('Password'),
      '#description' => $this->t('Password for authentication (if required). Leave empty to keep the current password.'),
    ];

    $form['test_connection'] = [
      '#type' => 'submit',
      '#value' => $this->t('Test Connection'),
      '#submit' => ['::testConnection'],
      '#limit_validation_errors' => [],
    ];

    return parent::buildForm($form, $form_state);
  }

    /**
    * Tests the connection to the OpenSearch server.
    */
    public function testConnection(array &$form, FormStateInterface $form_state) {
    // Get values from user input instead of processed values
    $values = $form_state->getUserInput();

    $host = $values['host'] ?? $this->config('opensearch_views.settings')->get('host') ?? 'localhost';
    $port = $values['port'] ?? $this->config('opensearch_views.settings')->get('port') ?? 9200;
    $username = $values['username'] ?? $this->config('opensearch_views.settings')->get('username');
    $password = $values['password'] ?? $this->config('opensearch_views.settings')->get('password');

    try {
        // Build host URL
        $hostUrl = 'http://' . $host . ':' . $port;

        // Use the builder pattern
        $clientBuilder = new \OpenSearch\ClientBuilder();
        $clientBuilder->setHosts([$hostUrl]);

        if (!empty($username) && !empty($password)) {
        $clientBuilder->setBasicAuthentication($username, $password);
        }

        $client = $clientBuilder->build();
        $info = $client->info();

        $this->messenger()->addStatus($this->t('Successfully connected to OpenSearch server. Version: @version', [
        '@version' => $info['version']['number'] ?? 'unknown',
        ]));
    }
    catch (\Exception $e) {
        $this->messenger()->addError($this->t('Failed to connect to OpenSearch server: @error', [
        '@error' => $e->getMessage(),
        ]));
    }
    }
    
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('opensearch_views.settings');
    $config->set('host', $form_state->getValue('host'));
    $config->set('port', $form_state->getValue('port'));
    $config->set('username', $form_state->getValue('username'));
    
    // Only update password if one was provided.
    if ($password = $form_state->getValue('password')) {
      $config->set('password', $password);
    }
    
    $config->save();

    parent::submitForm($form, $form_state);
  }
}
