<?php

namespace Drupal\opensearch_views;

use Drupal\Core\Config\ConfigFactoryInterface;
use OpenSearch\Client;
use OpenSearch\ClientBuilder;

/**
 * Service for interacting with OpenSearch.
 */
class OpenSearchClient {

  /**
   * The OpenSearch client.
   *
   * @var \OpenSearch\Client
   */
  protected $client;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Get the OpenSearch client.
   *
   * @return \OpenSearch\Client
   *   The OpenSearch client.
   */
    public function getClient() {
    if (!$this->client) {
        $config = $this->configFactory->get('opensearch_views.settings');

        // Ensure we have default values if config is empty
        $host = $config->get('host');
        if (empty($host)) {
        $host = 'localhost';
        }

        $port = $config->get('port');
        if (empty($port) || !is_numeric($port)) {
        $port = 9200;
        }

        // Build host URL (http://host:port)
        $hostUrl = 'http://' . $host . ':' . $port;

        // Use the builder pattern directly
        $clientBuilder = new \OpenSearch\ClientBuilder();
        $clientBuilder->setHosts([$hostUrl]);

        // Add authentication if provided
        $username = $config->get('username');
        $password = $config->get('password');
        if (!empty($username) && !empty($password)) {
        $clientBuilder->setBasicAuthentication($username, $password);
        }

        // Build the client
        $this->client = $clientBuilder->build();
    }

    return $this->client;
    }    

  /**
   * Search the OpenSearch index.
   *
   * @param string $index
   *   The index name.
   * @param array $params
   *   The search parameters.
   *
   * @return array
   *   The search results.
   */
  public function search($index, array $params = []) {
    $client = $this->getClient();
    
    $search_params = [
      'index' => $index,
      'body' => $params,
    ];
    
    return $client->search($search_params);
  }

  /**
   * Get mappings for an index.
   *
   * @param string $index
   *   The index name.
   *
   * @return array
   *   The mappings information.
   */
  public function getMappings($index) {
    $client = $this->getClient();
    
    try {
      return $client->indices()->getMapping(['index' => $index]);
    }
    catch (\Exception $e) {
      return [];
    }
  }

  /**
   * Check if an index exists.
   *
   * @param string $index
   *   The index name.
   *
   * @return bool
   *   TRUE if the index exists, FALSE otherwise.
   */
  public function indexExists($index) {
    $client = $this->getClient();
    
    try {
      return $client->indices()->exists(['index' => $index]);
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

    /**
    * Get a list of indices.
    *
    * @return array
    *   An array of index names.
    */
    public function getIndices() {
    try {
        $client = $this->getClient();

        // Log connection attempt
        \Drupal::logger('opensearch_views')->notice('Attempting to retrieve OpenSearch indices');

        $indices = $client->cat()->indices(['format' => 'json']);
        $result = [];

        foreach ($indices as $index) {
        $result[$index['index']] = $index['index'];
        }

        // Log success
        \Drupal::logger('opensearch_views')->notice('Successfully retrieved @count indices', ['@count' => count($result)]);

        return $result;
    }
    catch (\Exception $e) {
        // Log the error with detailed information
        \Drupal::logger('opensearch_views')->error('Failed to retrieve OpenSearch indices: @error', [
        '@error' => $e->getMessage()
        ]);

        // Return a message in the array to indicate failure
        return ['error' => t('Could not retrieve indexes. Check your connection settings.')];
    }
    }
  /**
   * Get document by ID.
   *
   * @param string $index
   *   The index name.
   * @param string $id
   *   The document ID.
   *
   * @return array|null
   *   The document or NULL if not found.
   */
  public function getDocument($index, $id) {
    $client = $this->getClient();
    
    try {
      $result = $client->get([
        'index' => $index,
        'id' => $id,
      ]);
      
      return $result;
    }
    catch (\Exception $e) {
      return NULL;
    }
  }
}
