<?php

namespace Drupal\opensearch_views\Plugin\views\field;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\opensearch_views\FieldManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Field handler for OpenSearch JSON fields.
 *
 * @ViewsField("opensearch_json_field")
 */
class OpenSearchJsonField extends FieldPluginBase {

  /**
   * The field manager service.
   *
   * @var \Drupal\opensearch_views\FieldManager
   */
  protected $fieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('opensearch_views.field_manager')
    );
  }

  /**
   * Constructs a new OpenSearchJsonField object.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, FieldManager $field_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fieldManager = $field_manager;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    
    $options['json_key_path'] = ['default' => ''];
    $options['format'] = ['default' => 'plain'];
    
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    
    // Get the index from the query or filter.
    $index = $this->getSelectedIndex();
    
    // Debug info - Log what index was found (or not)
    \Drupal::logger('opensearch_views')->notice('Building field options form. Result from getSelectedIndex: [@index]', [
      '@index' => $index ?? '(NULL)', // Use NULL explicitly if empty
    ]);

    // Get field options.
    $field_options = [];
    if (!empty($index)) {
      try {
         $field_options = $this->fieldManager->getFieldOptions($index);
          \Drupal::logger('opensearch_views')->notice('Field options count for index [@index]: @count', [
            '@index' => $index,
            '@count' => count($field_options),
          ]);
      } catch (\Exception $e) {
           // Log errors during field option fetching
           \Drupal::logger('opensearch_views')->error('Error fetching field options for index [@index]: @message', [
               '@index' => $index,
               '@message' => $e->getMessage(),
           ]);
      }
    }
    else {
      // Log the warning if index is still not found after trying harder
      \Drupal::logger('opensearch_views')->warning('No index found by getSelectedIndex, cannot populate field options.');
    }

    // If still no fields or only the default ones, try direct sampling (keep this fallback)
    if (count($field_options) <= 2 && !empty($index)) {
       \Drupal::logger('opensearch_views')->notice('Few options from FieldManager, attempting direct field discovery for index [@index]', ['@index' => $index]);
       $field_options = $this->getFieldOptionsDirectly($index); // Fallback depends on $index too
    }

    $form['json_key_path'] = [
      '#type' => 'select',
      '#title' => $this->t('JSON Field Path'),
      '#description' => $this->t('Select the JSON field to display.'),
      '#options' => $field_options, // Use the potentially populated options
      '#default_value' => $this->options['json_key_path'],
      '#empty_option' => $this->t('- Select a field -'),
       // Add AJAX refresh if needed? Usually handled by Views UI itself.
    ];    
    
    // Option to allow entering custom JSON path if not in the list.
    $form['custom_json_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom JSON Field Path'),
      '#description' => $this->t('Alternatively, enter a custom JSON path (dot notation, e.g. "dc:title" or "@graph.0.dc:description").'),
      '#default_value' => !in_array($this->options['json_key_path'], array_keys($field_options)) ? $this->options['json_key_path'] : '',
    ];
    
    $form['format'] = [
      '#type' => 'select',
      '#title' => $this->t('Format'),
      '#description' => $this->t('Select how to format complex values (arrays and objects).'),
      '#options' => [
        'plain' => $this->t('Plain text'),
        'json' => $this->t('JSON string'),
        'unformatted' => $this->t('Unformatted (debug)'),
      ],
      '#default_value' => $this->options['format'],
    ];
  }

  /**
   * Get the selected index - More Resilient Version.
   * Tries query options first, then iterates filters checking plugin ID and options.
   */
    protected function getSelectedIndex() {
        // Attempt 1: Try the potentially live query options first
        $view_query_options_index = $this->view->query->options['index'] ?? NULL;
        if (!empty($view_query_options_index)) {
             \Drupal::logger('opensearch_views')->info('getSelectedIndex (Attempt 1) found index [@index] in LIVE query options.', ['@index' => $view_query_options_index]);
             return $view_query_options_index;
        }

        // Attempt 2: Try the potentially live filter options directly
        if (!empty($this->view->filter)) {
            foreach ($this->view->filter as $filter_id => $filter) {
                if (is_object($filter) && isset($filter->options) && $filter->getPluginId() === 'opensearch_index_filter') {
                    $filter_options_index = $filter->options['index'] ?? NULL;
                    if (!empty($filter_options_index)) {
                         \Drupal::logger('opensearch_views')->info('getSelectedIndex (Attempt 2) found index [@index] in LIVE filter options (ID: @filter_id).', ['@index' => $filter_options_index, '@filter_id' => $filter_id]);
                         return $filter_options_index;
                    }
                    // Found the filter, but its option was empty in the live object state
                    \Drupal::logger('opensearch_views')->notice('getSelectedIndex (Attempt 2) found LIVE index filter [@filter_id], but its index value is empty.', ['@filter_id' => $filter_id]);
                    break; // Stop checking live filters
                }
            }
        } else {
             \Drupal::logger('opensearch_views')->notice('getSelectedIndex (Attempt 2) skipped: $this->view->filter was empty.');
        }

        // --- Attempt 3: Load SAVED View Configuration ---
        \Drupal::logger('opensearch_views')->info('getSelectedIndex: Live options failed, attempting to load saved view config for view [@view_id], display [@display_id].', [
            '@view_id' => $this->view->storage->id() ?? 'N/A',
            '@display_id' => $this->view->current_display ?? 'N/A',
        ]);

        try {
            // Ensure view storage and display ID are available
            if (empty($this->view->storage) || empty($this->view->current_display)) {
                 \Drupal::logger('opensearch_views')->warning('getSelectedIndex (Attempt 3): Cannot load saved config - view storage or current display is missing from $this->view.');
                 return NULL;
            }

            // Get the raw configuration array for the current display from the View entity
            $display_config = $this->view->storage->getDisplay($this->view->current_display);

            // Check if filters configuration exists in the saved display options
            if (isset($display_config['display_options']['filters']) && is_array($display_config['display_options']['filters'])) {
                // Iterate through the filters defined in the *saved configuration*
                foreach ($display_config['display_options']['filters'] as $filter_config_id => $filter_config) {
                    // Check if this filter in the *saved config* is the index filter
                    if (isset($filter_config['plugin_id']) && $filter_config['plugin_id'] === 'opensearch_index_filter') {
                         // Extract the saved index value directly from the configuration array
                         $saved_index = $filter_config['index'] ?? NULL;
                        if (!empty($saved_index)) {
                             // Success! Found the index in the saved configuration.
                             \Drupal::logger('opensearch_views')->info('getSelectedIndex (Attempt 3 - SUCCESS) found index [@index] in SAVED view config (Filter ID: @filter_config_id).', ['@index' => $saved_index, '@filter_config_id' => $filter_config_id]);
                             return $saved_index;
                        }
                         else {
                             // Found the filter config, but the index value itself is empty in the saved config.
                             \Drupal::logger('opensearch_views')->warning('getSelectedIndex (Attempt 3) found SAVED index filter [@filter_config_id], but its index value is empty.', ['@filter_config_id' => $filter_config_id]);
                             break; // Stop checking saved filters
                         }
                    }
                }
                // If the loop finished without finding the filter config
                 \Drupal::logger('opensearch_views')->warning('getSelectedIndex (Attempt 3) iterated SAVED filters but did not find opensearch_index_filter config with a value.');
            } else {
                 // The saved display options array doesn't contain a filters key.
                 \Drupal::logger('opensearch_views')->warning('getSelectedIndex (Attempt 3) could not find "filters" array key in SAVED display options.');
            }
        } catch (\Exception $e) {
             // Log any error during config loading or processing
             \Drupal::logger('opensearch_views')->error('getSelectedIndex (Attempt 3) Exception loading/reading saved view config for view [@view_id], display [@display_id]: @message', [
                '@view_id' => $this->view->storage->id() ?? 'N/A',
                '@display_id' => $this->view->current_display ?? 'N/A',
                '@message' => $e->getMessage()
             ]);
        }


        // If ALL attempts failed
        \Drupal::logger('opensearch_views')->error('getSelectedIndex: ALL attempts failed to find the index.');
        return NULL;
    }    
  /**
   * Get field options directly from a sample document.
   */
  protected function getFieldOptionsDirectly($index) {
    $options = [
      '_id' => 'Document ID (_id)',
      '_source' => 'Full Document (_source)',
    ];
    
    if (empty($index)) {
      return $options;
    }
    
    try {
      // Get client directly
      $client = \Drupal::service('opensearch_views.client');
      $search_result = $client->search($index, ['size' => 1]);
      
      if (!empty($search_result['hits']['hits'])) {
        $document = $search_result['hits']['hits'][0]['_source'] ?? [];
        
        // Extract top-level fields first
        foreach ($document as $key => $value) {
          $type = $this->getValueType($value);
          $options[$key] = $key . ' (' . $type . ')';
          
          // Special handling for common nested structures in your JSON
          if ($key === '@graph' && is_array($value)) {
            // Handle @graph array
            foreach ($value as $graphIndex => $graphItem) {
              $graphPath = '@graph.' . $graphIndex;
              $options[$graphPath] = $graphPath . ' (object)';
              
              // Extract fields from each graph item
              if (is_array($graphItem) || is_object($graphItem)) {
                foreach ($graphItem as $graphKey => $graphValue) {
                  $nestedPath = $graphPath . '.' . $graphKey;
                  $options[$nestedPath] = $nestedPath . ' (' . $this->getValueType($graphValue) . ')';
                  
                  // Go one level deeper for common structures
                  if ((is_array($graphValue) || is_object($graphValue)) && !empty($graphValue)) {
                    foreach ((array)$graphValue as $deepKey => $deepValue) {
                      $deepPath = $nestedPath . '.' . $deepKey;
                      $options[$deepPath] = $deepPath . ' (' . $this->getValueType($deepValue) . ')';
                    }
                  }
                }
              }
            }
          }
          // Handle dc: and other namespace prefixes
          else if (strpos($key, ':') !== false && (is_array($value) || is_object($value))) {
            foreach ((array)$value as $nsKey => $nsValue) {
              $nsPath = $key . '.' . $nsKey;
              $options[$nsPath] = $nsPath . ' (' . $this->getValueType($nsValue) . ')';
            }
          }
        }
        
        // Sort options alphabetically
        ksort($options);
      }
    }
    catch (\Exception $e) {
      \Drupal::logger('opensearch_views')->error('Error in direct field discovery: @error', [
        '@error' => $e->getMessage(),
      ]);
    }
    
    return $options;
  }

  /**
   * Get the value type.
   */
  protected function getValueType($value) {
    if (is_null($value)) {
      return 'null';
    }
    if (is_bool($value)) {
      return 'boolean';
    }
    if (is_int($value)) {
      return 'integer';
    }
    if (is_float($value)) {
      return 'float';
    }
    if (is_string($value)) {
      return 'string';
    }
    if (is_array($value)) {
      return isset($value[0]) ? 'array' : 'object';
    }
    if (is_object($value)) {
      return 'object';
    }
    return 'unknown';
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    $values = $form_state->getValue(['options']);
    
    // Use custom path if provided.
    if (!empty($values['custom_json_path'])) {
      $form_state->setValue(['options', 'json_key_path'], $values['custom_json_path']);
    }
    
    parent::submitOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $row) {
    // Get the JSON path.
    $path = $this->options['json_key_path'];
    if (empty($path)) {
      return '';
    }
    
    // Special handling for full source document
    if ($path === '_source') {
      return $this->formatValue($row->opensearch_source);
    }
    
    // Special handling for document ID
    if ($path === '_id') {
      return $row->opensearch_id ?? '';
    }
    
    // Extract the value from the document.
    $source = $row->opensearch_source;
    $value = $this->getNestedValue($source, $path);
    
    // Format the value based on its type and the selected format.
    return $this->formatValue($value);
  }

  /**
   * Get a nested value from an array using dot notation.
   *
   * @param array $array
   *   The array to extract the value from.
   * @param string $path
   *   The path to the value using dot notation.
   *
   * @return mixed
   *   The value or NULL if not found.
   */
  protected function getNestedValue(array $array, $path) {
    // Handle array index notation like 'items[0].name'
    $path = preg_replace_callback('/\[(\d+)\]/', function($matches) {
      return '.' . $matches[1];
    }, $path);
    
    $keys = explode('.', $path);
    $current = $array;
    
    foreach ($keys as $key) {
      // Handle special characters in field names
      if (strpos($key, ':') !== false) {
        // This handles namespace prefixes like dc:title
        $key = str_replace('\\:', ':', $key); // Remove any escape slashes
      }
      
      // Skip if the current element is not an array or the key doesn't exist
      if (!is_array($current) || !isset($current[$key])) {
        return NULL;
      }
      
      $current = $current[$key];
    }
    
    return $current;
  }

  /**
   * Format a value based on its type and the selected format option.
   *
   * @param mixed $value
   *   The value to format.
   *
   * @return string
   *   The formatted value.
   */
  protected function formatValue($value) {
    if ($value === NULL) {
      return '';
    }
    
    if (is_scalar($value)) {
      return (string) $value;
    }
    
    // Handle arrays and objects.
    switch ($this->options['format']) {
      case 'json':
        return json_encode($value, JSON_PRETTY_PRINT);
        
      case 'unformatted':
        return '<pre>' . print_r($value, TRUE) . '</pre>';
        
      case 'plain':
      default:
        if (is_array($value)) {
          // For simple arrays, concatenate values.
          if (count($value) === 0) {
            return '';
          }
          
          // Check if this is a simple array of scalars.
          $is_simple = TRUE;
          foreach ($value as $item) {
            if (!is_scalar($item)) {
              $is_simple = FALSE;
              break;
            }
          }
          
          if ($is_simple) {
            return implode(', ', $value);
          }
          
          // For complex arrays, use JSON format.
          return json_encode($value, JSON_PRETTY_PRINT);
        }
        
        // Default for objects and other types.
        return json_encode($value, JSON_PRETTY_PRINT);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // We don't need to add anything to the query, as we're relying on
    // retrieving the full document and extracting values in render().
  }
}
