<?php

namespace Drupal\opensearch_views\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\filter\FilterPluginBase;
use Drupal\opensearch_views\OpenSearchClient;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Filter handler for OpenSearch index selection.
 *
 * @ViewsFilter("opensearch_index_filter")
 */
class OpenSearchIndexFilter extends FilterPluginBase {

  /**
   * The OpenSearch client.
   *
   * @var \Drupal\opensearch_views\OpenSearchClient
   */
  protected $client;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('opensearch_views.client')
    );
  }

  /**
   * Constructs a new OpenSearchIndexFilter object.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, OpenSearchClient $client) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->client = $client;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['index'] = ['default' => ''];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    
    // Get all available indices from OpenSearch.
    $indices = $this->client->getIndices();
    
    // Filter out error message if present.
    if (isset($indices['error'])) {
      $indices = [];
    }
    
    $form['index'] = [
      '#type' => 'select',
      '#title' => $this->t('OpenSearch Index'),
      '#description' => $this->t('Select the OpenSearch index to query.'),
      '#options' => $indices,
      '#default_value' => $this->options['index'],
      '#required' => TRUE,
      '#empty_option' => $this->t('- Select an index -'),
    ];
    
    // Hide the operator and value fields as they are not needed.
    $form['operator']['#access'] = FALSE;
    $form['value']['#access'] = FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    // Instead of adding a filter, we set the index on the query.
    if (!empty($this->options['index'])) {
      $this->query->options['index'] = $this->options['index'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function adminSummary() {
    return $this->options['index'];
  }

  /**
   * {@inheritdoc}
   */
  public function canExpose() {
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    // Override to avoid checking for an operator and a value.
  }
}
