<?php

namespace Drupal\opensearch_views\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\filter\StringFilter;
use Drupal\opensearch_views\FieldManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Filter handler for OpenSearch JSON fields.
 *
 * @ViewsFilter("opensearch_json_filter")
 */
class OpenSearchJsonFilter extends StringFilter {

  /**
   * The field manager service.
   *
   * @var \Drupal\opensearch_views\FieldManager
   */
  protected $fieldManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('opensearch_views.field_manager')
    );
  }

  /**
   * Constructs a new OpenSearchJsonFilter object.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, FieldManager $field_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fieldManager = $field_manager;
  }

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    
    $options['json_key_path'] = ['default' => ''];
    
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    
    // Get the index from the query.
    $index = '';
    if (isset($this->view->query->options['index'])) {
      $index = $this->view->query->options['index'];
    }
    
    // Get field options.
    $field_options = [];
    if (!empty($index)) {
      $field_options = $this->fieldManager->getFieldOptions($index);
    }
    
    $form['json_key_path'] = [
      '#type' => 'select',
      '#title' => $this->t('JSON Field Path'),
      '#description' => $this->t('Select the JSON field to filter on.'),
      '#options' => $field_options,
      '#default_value' => $this->options['json_key_path'],
      '#empty_option' => $this->t('- Select a field -'),
    ];
    
    // Option to allow entering custom JSON path if not in the list.
    $form['custom_json_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom JSON Field Path'),
      '#description' => $this->t('Alternatively, enter a custom JSON path (dot notation).'),
      '#default_value' => !in_array($this->options['json_key_path'], array_keys($field_options)) ? $this->options['json_key_path'] : '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitOptionsForm(&$form, FormStateInterface $form_state) {
    $values = $form_state->getValue(['options']);
    
    // Use custom path if provided.
    if (!empty($values['custom_json_path'])) {
      $form_state->setValue(['options', 'json_key_path'], $values['custom_json_path']);
    }
    
    parent::submitOptionsForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $this->ensureMyTable();
    
    if (empty($this->options['json_key_path'])) {
      return;
    }
    
    $field = $this->options['json_key_path'];
    
    // Map operator to OpenSearch query types.
    $operator = $this->operator;
    
    if ($operator === 'contains') {
      $operator = 'contains';
    }
    elseif ($operator === 'starts') {
      $operator = 'starts';
    }
    elseif ($operator === 'ends') {
      $operator = 'ends';
    }
    elseif ($operator === 'word') {
      // For word boundary, use contains.
      $operator = 'contains';
    }
    
    $this->query->addWhere($this->options['group'], $field, $this->value, $operator);
  }
}
