<?php

namespace Drupal\opensearch_views\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\filter\StringFilter;

/**
 * Filter handler for exposed keyword search on OpenSearch.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("opensearch_keyword_filter")
 */
class OpenSearchKeywordFilter extends StringFilter {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    // Option to specify which fields to search. Comma-separated.
    $options['search_fields'] = ['default' => ''];
    // Option for the query operator (AND/OR) for simple_query_string.
    $options['default_operator'] = ['default' => 'OR'];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['search_fields'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search Fields'),
      '#description' => $this->t('Optional: Comma-separated list of fields to search (e.g., "title, body, @graph.*.dc:description"). Supports wildcards like *. Leave empty to use OpenSearch default fields or query across all allowed fields (depending on OpenSearch configuration and query type).'),
      '#default_value' => $this->options['search_fields'],
    ];

    $form['default_operator'] = [
        '#type' => 'radios',
        '#title' => $this->t('Default Operator'),
        '#options' => ['OR' => $this->t('OR'), 'AND' => $this->t('AND')],
        '#description' => $this->t('The default operator for the query terms if none is specified by the user (e.g., searching for "red car" with OR matches documents with "red" or "car"; with AND requires both). Applies to simple_query_string.'),
        '#default_value' => $this->options['default_operator'],
    ];
  }

  /**
   * {@inheritdoc}
   *
   * Modify the main OpenSearch query body, not the filter context.
   */
    public function query() {
        // Only add the clause if the user actually entered a value.
        if (!empty($this->value)) {
            // Use simple_query_string for robust user input handling.
            $query_clause = [
                'simple_query_string' => [
                    'query' => $this->value,
                    'default_operator' => $this->options['default_operator'],
                    // Add other options as needed, like 'analyze_wildcard' => TRUE
                ],
            ];

            // Add specific fields if configured.
            $fields_string = trim($this->options['search_fields']);
            if (!empty($fields_string)) {
                // Split comma-separated string and trim whitespace from each field.
                $fields_array = array_map('trim', explode(',', $fields_string));
                // Filter out any potentially empty values after trimming/exploding.
                $fields_array = array_filter($fields_array);
                if (!empty($fields_array)) {
                    $query_clause['simple_query_string']['fields'] = $fields_array;
                }
            }

            // *** MODIFICATION START: Use public method to add the clause ***
            // Add the clause to the 'must' section using the query plugin's method.
            // Check if the method exists on the query handler object first for safety.
            if (method_exists($this->query, 'addQueryClause')) {
                 // Add to 'must' - requires keywords to match (AND logic overall).
                 // Change 'must' to 'should' if keywords should broaden results (OR logic overall).
                 $this->query->addQueryClause('must', $query_clause);
            } else {
                // Log an error if the query plugin is missing the expected method.
                 \Drupal::logger('opensearch_views')->error('The OpenSearchViewsQuery object is missing the required addQueryClause method.');
                 // You might want to display a message to the user here as well.
            }
            // *** MODIFICATION END ***
        }
    } // End query()

}
