<?php

namespace Drupal\opensearch_views\Plugin\views\query;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\query\QueryPluginBase;
use Drupal\views\ResultRow;
use Drupal\views\ViewExecutable;
use Drupal\opensearch_views\OpenSearchClient;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * OpenSearch Views query plugin.
 *
 * @ViewsQuery(
 *   id = "opensearch_views_query",
 *   title = @Translation("OpenSearch Query"),
 *   help = @Translation("Query against OpenSearch indices.")
 * )
 */
class OpenSearchViewsQuery extends QueryPluginBase {

  /**
   * The OpenSearch client.
   *
   * @var \Drupal\opensearch_views\OpenSearchClient
   */
  protected $client;

  /**
   * Query parameters.
   *
   * @var array
   */
  protected $query = [];

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('opensearch_views.client')
    );
  }

  /**
   * Constructs a OpenSearchViewsQuery object.
   */
    public function __construct(array $configuration, $plugin_id, $plugin_definition, OpenSearchClient $client) {
        parent::__construct($configuration, $plugin_id, $plugin_definition);
        $this->client = $client;

        // *** ADD INITIALIZATION HERE ***
        // Initialize the query structure early to prevent errors/warnings
        // if handlers access it before build() fully completes.
        $this->query = [
            'query' => [
                'bool' => [
                    'must' => [],
                    'should' => [],
                    'must_not' => [],
                    'filter' => [],
                ],
            ],
            // Initialize other keys that might be expected later if needed
            'size' => 10, // Default size
            'sort' => [], // Default sort
        ];
        // Note: build() will overwrite/add to this structure later based on options.
    }

  /**
   * {@inheritdoc}
   */
  public function defineOptions() {
    $options = parent::defineOptions();
    
    $options['index'] = [
      'default' => '',
    ];
    
    $options['query_string'] = [
      'default' => '',
    ];
    
    $options['size'] = [
      'default' => 10,
    ];
    
    return $options;
  }


  /**
   * {@inheritdoc}
   */
    public function build(ViewExecutable $view) {
        $this->view = $view;

        // Start with the base structure (already initialized in constructor)
        // Ensure critical keys exist if constructor didn't set them somehow
         if (!isset($this->query['query']['bool'])) {
             $this->query['query']['bool'] = [
                 'must' => [], 'should' => [], 'must_not' => [], 'filter' => [],
             ];
         }
         if (!isset($this->query['query']['bool']['must'])) { $this->query['query']['bool']['must'] = []; }


        // Set size based on options, overwriting default from constructor.
        $this->query['size'] = $this->options['size'] ?? 10;

        // Only add 'from' parameter if offset is valid and > 0
        if (isset($this->offset) && is_numeric($this->offset) && $this->offset > 0) {
           $this->query['from'] = $this->offset;
        } else {
            // Ensure 'from' is removed if offset is not applicable (unset might be safer than setting to 0)
            unset($this->query['from']);
        }


        // Add query string from options (if provided) to the 'must' clause
        if (!empty($this->options['query_string'])) {
            $this->query['query']['bool']['must'][] = [
                'query_string' => [
                    'query' => $this->options['query_string'],
                ],
            ];
        }

        // Handle sorting
        if (!empty($this->orderby) && is_array($this->orderby)) {
            $this->query['sort'] = []; // Initialize/reset sort array
            foreach ($this->orderby as $field => $order) {
                 // Ensure order is a valid string ('asc' or 'desc')
                 $order = strtolower($order) === 'desc' ? 'desc' : 'asc';
                 $this->query['sort'][] = [$field => ['order' => $order]];
            }
        } else {
            // Ensure 'sort' is removed if no orderby is set
            unset($this->query['sort']);
        }


        // *** Let Views call handlers (like filter->query()) which might call addQueryClause ***
        // This happens implicitly as part of ViewExecutable::build()


        // Allow modules to modify the query *after* handlers have run.
        $this->alterQuery();
    }

  /**
   * {@inheritdoc}
   */
  public function execute(ViewExecutable $view) {
    $start = microtime(TRUE);

    // Build the query.
    $this->build($view);

    // Execute the search.
    try {
      $index = $this->options['index'] ?? '';
      if (empty($index)) {
        $view->result = [];
        $view->total_rows = 0;
        $view->execute_time = microtime(TRUE) - $start;
        return;
      }

      $results = $this->client->search($index, $this->query);

      // Process the results.
      $view->result = [];
      $view->total_rows = $results['hits']['total']['value'] ?? 0;
      
      // Check if pager exists before setting properties on it
      if (isset($view->pager) && is_object($view->pager)) {
          $view->pager->total_items = $view->total_rows;
          $view->pager->updatePageInfo();
      }
      
      // Convert OpenSearch results to Views result rows.
      $index = 0;
      foreach ($results['hits']['hits'] as $hit) {
        $row = new ResultRow();
        $row->index = $index++;
        $row->opensearch_id = $hit['_id'];
        $row->opensearch_score = $hit['_score'] ?? 0;
        $row->opensearch_source = $hit['_source'];
        $view->result[] = $row;
      }
    }
    catch (\Exception $e) {
      // Log the error and set an empty result.
      \Drupal::logger('opensearch_views')->error('Error executing OpenSearch query: @error', [
        '@error' => $e->getMessage(),
      ]);
      
      $view->result = [];
      $view->total_rows = 0;
    }

    $view->execute_time = microtime(TRUE) - $start;
  }

  /**
   * Add a field to the query.
   */
  public function addField($table, $field, $alias = '', $params = []) {
    // We don't need to actually add fields to the query since we're fetching
    // the entire document from OpenSearch.
    return $field;
  }

  /**
   * Add a filter to the query.
   */
  public function addWhere($group, $field, $value = NULL, $operator = NULL) {
    // Map the operator to an OpenSearch query type.
    $filter = [];
    
    switch ($operator) {
      case '=':
        $filter = ['term' => [$field => $value]];
        break;
        
      case '!=':
        $filter = ['bool' => ['must_not' => ['term' => [$field => $value]]]];
        break;
        
      case '>':
        $filter = ['range' => [$field => ['gt' => $value]]];
        break;
        
      case '>=':
        $filter = ['range' => [$field => ['gte' => $value]]];
        break;
        
      case '<':
        $filter = ['range' => [$field => ['lt' => $value]]];
        break;
        
      case '<=':
        $filter = ['range' => [$field => ['lte' => $value]]];
        break;
        
      case 'contains':
        $filter = ['wildcard' => [$field => "*$value*"]];
        break;
        
      case 'starts':
        $filter = ['wildcard' => [$field => "$value*"]];
        break;
        
      case 'ends':
        $filter = ['wildcard' => [$field => "*$value"]];
        break;
        
      case 'in':
        $filter = ['terms' => [$field => $value]];
        break;
        
      case 'not in':
        $filter = ['bool' => ['must_not' => ['terms' => [$field => $value]]]];
        break;
        
      default:
        // Default to exact match.
        $filter = ['term' => [$field => $value]];
        break;
    }
    
    // Add the filter to the appropriate group (must, should, must_not).
    if (!empty($filter)) {
      $this->query['query']['bool']['filter'][] = $filter;
    }
  }

  /**
   * Add a simple condition to the query.
   */
  public function addCondition($field, $value, $operator = '=') {
    $this->addWhere(0, $field, $value, $operator);
  }

  /**
   * Add a sort to the query.
   */
  public function addOrderBy($table, $field, $order = 'ASC', $alias = '', $params = []) {
    $this->orderby[$field] = strtolower($order);
  }


   /**
     * Adds an arbitrary query clause to a specific part of the boolean query.
     *
     * This provides a public interface for other plugins (like filters)
     * to add clauses to the main query body (must, should, must_not)
     * instead of only the filter context (via addWhere).
     *
     * @param string $booleanClauseType
     * The type of boolean clause ('must', 'should', 'must_not', 'filter').
     * @param array $clause
     * The OpenSearch query clause array (e.g., ['term' => ...], ['multi_match' => ...]).
     */
    public function addQueryClause(string $booleanClauseType, array $clause) {
        // *** MODIFICATION: Remove the warning log, but keep the safety check/init ***
        // Ensure the main query structure is initialized. Although constructor should do this,
        // keep the check for absolute safety in case build() resets $this->query somehow.
        if (!isset($this->query['query']['bool'])) {
            $this->query['query']['bool'] = [
                'must' => [], 'should' => [], 'must_not' => [], 'filter' => [],
            ];
             // Warning removed - constructor should prevent this now.
        }

        // Validate the clause type.
        $validClauseTypes = ['must', 'should', 'must_not', 'filter'];
        if (!in_array($booleanClauseType, $validClauseTypes)) {
             \Drupal::logger('opensearch_views')->error('Invalid boolean clause type "@type" passed to addQueryClause.', ['@type' => $booleanClauseType]);
             return;
        }

         // Ensure the specific clause array exists (e.g., $this->query['query']['bool']['must']).
         if (!isset($this->query['query']['bool'][$booleanClauseType])) {
             $this->query['query']['bool'][$booleanClauseType] = [];
         }

         // Add the clause.
         $this->query['query']['bool'][$booleanClauseType][] = $clause;
    }    
    
  /**
   * Alter the query before it is executed.
   */
  protected function alterQuery() {
    \Drupal::moduleHandler()->alter('opensearch_views_query', $this->query, $this->view);
  }
}
